#ifndef _ARGP_FMTSTREAM_H
#include <argp/argp-fmtstream.h>

#ifndef _ISOMAC
extern __typeof (__argp_fmtstream_ensure) __argp_fmtstream_ensure
      attribute_hidden;
extern __typeof (__argp_fmtstream_free) __argp_fmtstream_free
      attribute_hidden;
extern __typeof (__argp_fmtstream_printf) __argp_fmtstream_printf
      attribute_hidden;
extern __typeof (__argp_fmtstream_update) __argp_fmtstream_update
      attribute_hidden;
extern __typeof (__argp_fmtstream_write) __argp_fmtstream_write
      attribute_hidden;
extern __typeof (__argp_make_fmtstream) __argp_make_fmtstream
      attribute_hidden;
#endif

#endif
